import pickle
from magictk import photoload
from magictk import color_tmpl


def set_color_style(color_set={}, **kwargs):
    if color_set == {}:
        color_tmpl.default_color.update(kwargs)
    else:
        color_tmpl.default_color.update(color_set)


def load_icon_pack(path):
    with open(path, "rb") as file:
        photoload.image_all.update(pickle.load(file))



import json
import tkinter
from tkinter import ttk

from magictk import color_tmpl
from magictk import photoload
from magictk import fontconfig


class Icon:
    color = color_tmpl.default_color
    size = 12
    iconname = "apple"
    setcolor = "primary"

    def _draw(self, x, y, **kwargs):
        border_info = json.loads(photoload.loadres(
            f"icon/{self.iconname}@{self.size}"))
        y_n = 0
        for i in border_info:
            x_n = 0
            for j in i:
                px = x+x_n+1
                py = y+y_n+1
                if (j < 0):
                    lcolor = -j
                else:
                    lcolor = j
                if (self.setcolor[0] == '#'):
                    g_color = color_tmpl.mix_color(
                        self.setcolor, self.color["background"], int((1-lcolor/255)*1000)/1000)
                else:
                    g_color = color_tmpl.mix_color(
                        self.color[self.setcolor], self.color["background"], int((1-lcolor/255)*1000)/1000)

                obj = self.canvas.create_rectangle(
                    py, px, py, px, width=0, fill=g_color)
                x_n += 1
            y_n += 1

    def __init__(self, master=None, size=16, iconname="apple", setcolor="regular_text", color_list: dict = None, _set_defaultcolor=None):
        self.size = size
        self.iconname = iconname
        self.setcolor = setcolor
        self.canvas = tkinter.Canvas(
            master, bg=self.color["background"], width=size+2, height=size+2, borderwidth=0, bd=0, highlightcolor=self.color["background"], highlightthickness=0)

        self._draw(0, 0)

    def pack(self, *args, **kwargs):
        self.canvas.pack(*args, **kwargs)

    def grid(self, *args, **kwargs):
        self.canvas.grid(*args, **kwargs)

    def place(self, *args, **kwargs):
        self.canvas.place(*args, **kwargs)



import tkinter
from tkinter import ttk
from magictk import color_tmpl
from magictk.scrollbar import ScrollBar
import sys


class Frame(tkinter.Frame):
    color = color_tmpl.default_color

    def __init__(self, master, color=None, *args, **kwargs):
        self.root = master.root
        self.p_event_list = ()
        if ("bg" not in kwargs and "background" not in kwargs):
            kwargs["bg"] = self.color["background"]
        super().__init__(master, *args, **kwargs)
        self.configure()

        for event, eventfunc in master.p_event_list:
            self.bind(event, eventfunc)


class Container(Frame):
    color = color_tmpl.default_color

    def scroll_callback(self, obj, pos):
        super().place(x=0, y=-pos, w=self.w-7, h=self.container_h)

    def update_frame(self, *args, **kwargs):
        self.w = self.root_frame.winfo_width()
        self.h = self.root_frame.winfo_height()
        super().place(x=0, y=-self.scroll.get(), w=self.w-7, h=self.container_h)
        self.scroll._r_allh = self.container_h
        self.scroll._r_maxh = self.h
        self.scroll.place(x=self.w-7, y=0, w=8, relheight=1)
        self.scroll.h = self.h
        scrolltmp = self.scroll.scroll_y*self.scroll.y_size
        self.scroll.cal_bar()
        self.scroll.update_pos()
        if (self.scroll.y_size == 0):
            self.scroll.scroll_y = 0
        else:
            self.scroll.scroll_y = min(self.scroll.h-self.scroll.bar_len,
                                       max(0, scrolltmp/self.scroll.y_size))
        self.scroll_callback(self.scroll, self.scroll.get())

    def scrollwheel(self, event: tkinter.Event, setdelta=None):
        if (setdelta is None):
            delta = event.delta
        else:
            delta = setdelta
        if sys.platform.startswith('win'):
            delta //= 20
        self.scroll.scroll_y = max(
            0, min(self.scroll._r_maxh-self.scroll.bar_len, self.scroll.scroll_y+delta))
        self.scroll.update_pos()
        self.scroll_callback(self.scroll, self.scroll.get())

    def __init__(self, master, color=None, w=300, h=200, container_h=500, *args, **kwargs):
        self.root = master.root
        self.w = w
        self.h = h
        self.container_h = container_h
        self.root_frame = Frame(master, width=w, height=self.container_h)
        super().__init__(self.root_frame, *args, **kwargs)
        super().place(x=0, y=0, w=self.w-7, h=container_h)
        self.scroll = ScrollBar(self.root_frame,
                                h=self.h, allh=self.container_h, maxh=self.h, callback=self.scroll_callback)
        self.scroll.place(x=self.w-7, y=0, w=8, relheight=1)
        self.root_frame.bind("<Configure>", self.update_frame)

        if sys.platform.startswith('win'):
            self.bind("<MouseWheel>", self.scrollwheel)
            self.p_event_list = (("<MouseWheel>", self.scrollwheel),)
        elif sys.platform.startswith('darwin'):
            self.bind("<MouseWheel>", self.scrollwheel)
            self.p_event_list = (("<MouseWheel>", self.scrollwheel),)
        else:
            self.bind(
                "<Button-4>", lambda event: self.scrollwheel(event, -10))
            self.bind(
                "<Button-5>", lambda event: self.scrollwheel(event, 10))
            self.p_event_list = (
                ("<Button-4>", lambda event: self.scrollwheel(event, -10)), ("<Button-5>", lambda event: self.scrollwheel(event, 10)))
        # self.configure

    def pack(self, *args, **kwargs):
        self.root_frame.pack(*args, **kwargs)

    def grid(self, *args, **kwargs):
        self.root_frame.grid(*args, **kwargs)

    def place(self, *args, **kwargs):
        self.root_frame.place(*args, **kwargs)
